(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_structural_mapping.mli,v 1.13 2014-01-21 17:17:21 fclement Exp $ *)

(** {3 Matrix structural mappings} *)

type 'a mapping = 'a Umatrix_graph.t;;

type block_number = Udelayed_dgraph.arrow_number;;

type ('a, 'b) block = ('a mapping, 'b) Udelayed_dgraph.delayed_arrow;;

type ('a, 'b, 'c) t = ('a mapping, 'b, 'c) Udelayed_dgraph.t;;
(** Blocks are meant to be supported by disjoint sub-domains, and the
 disjoint union of all sub-domains is meant to be the full universe. *)

val same : ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> bool;;

(** {6 Access functions} *)

val get : ('a, 'b, 'c) t -> block_number -> ('a, 'b) block;;
val set_next : ('a, 'b, 'c) t -> 'a mapping -> ('a, 'b, 'c) t;;
val set : ('a, 'b, 'c) t -> block_number -> 'a mapping -> ('a, 'b, 'c) t;;

val last_block_number : ('a, 'b, 'c) t -> block_number;;

(** {6 Converters} *)

val to_graph : ('a, 'b, 'c) t -> 'a mapping;;
val of_graph :
  ('c -> block_number -> 'a mapping -> 'b) -> 'a mapping -> ('a, 'b, 'c) t
;;

(** {6 Iterators} *)

val iter :
  (block_number -> ('a, 'b) block -> unit) ->
  ('a, 'b, 'c) t -> unit
;;

val fold_left :
  ('accu -> block_number -> ('a, 'b) block -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;

val iter_to_do :
  (block_number -> 'a mapping -> unit) ->
  ('a, 'b, 'c) t -> unit
;;

val iter_value :
  (block_number -> 'a mapping -> unit) ->
  ('a, 'b, 'c) t -> unit
;;

val fold_left_value :
  ('accu -> block_number -> 'a mapping -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;
