(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udynamic_graph.ml,v 1.7 2014/01/23 14:14:17 fclement Exp $ *)

(* Dynamic graphs. *)

module Sdarray = Ustamped_darray;;

type length = Sdarray.length;;
type arrow_number = Sdarray.index;;

type ('a, 'b) arrow =
  {
   value : 'a;
   image : 'b;
  }
;;

let get_value ar = ar.value;;

type ('a, 'b) t = ('a, 'b) arrow Ustamped_darray.t;;

let make = Sdarray.make;;

let get = Sdarray.get;;
let set = Sdarray.set;;

let next_unset dg = dg.Sdarray.next_unset;;
let set_next dg = set dg (next_unset dg);;

let same = Sdarray.same;;

(* Converters. *)

let to_array = Sdarray.to_array;;

let to_values dg = Array.map get_value (to_array dg);;

let arrow f x = { value = x; image = f x; };;

let of_values f a =
  let aa = Array.map (arrow f) a in
  Sdarray.of_array aa
;;

(* Iterators. *)

let iter = Sdarray.iter;;
let map = Sdarray.map;;
let fold_left = Sdarray.fold_left;;
