(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upositive_rational_range.ml,v 1.4 2014/02/09 17:27:28 fclement Exp $ *)

(* Positive rational number ranges. *)

let prr_qual = Upervasives.qualify "Upositive_rational_range";;

let prr_failwith = Upervasives.failwith_qual prr_qual;;

open Upositive_rational;;
open Upositive_rational_range_types;;

type t = Upositive_rational_range_types.t;;

let of_rat r1 r2 =
  try Nonempty (Upositive_rational_range_types.to_nonempty_range r1 r2) with
  | Failure _ -> Empty
;;

let intersection rg1 rg2 =
  match (rg1, rg2) with
  | (Empty, _) | (_, Empty) -> Empty
  | (Nonempty { left = l1; right = r1; },
     Nonempty { left = l2; right = r2; }) ->
       of_rat (Upositive_rational.max l1 l2) (Upositive_rational.min r1 r2)
;;

let project rg x =
  match rg with
  | Empty -> prr_failwith "project" "empty range"
  | Nonempty { left = l; right = r; } ->
      if smaller_or_equal l x && smaller_or_equal x r
      then x
      else if smaller r x then r
      else l
;;
