(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: usubarray_types.ml,v 1.9 2014/02/09 17:27:28 fclement Exp $ *)

(* The private type of subarrays. *)

let sa_qual = Upervasives.qualify "Usubarray";;

let sa_invalid_arg = Upervasives.invalid_arg_qual sa_qual;;

type index = Upervasives.index;;

type offset = Unatural.t
and length = Upervasives.length
;;

type range =
  {
   offset : offset;
   length : length;
  }
;;

type indices =
  | Range of range
  | Indices of index array
;;

type 'a t =
  {
   support : 'a array;
   indices : indices;
  }
;;

let range sup ({ offset = ofs; length = len; } as rng) =
  if Unatural.add ofs len > Uarray.length sup then
    sa_invalid_arg "range" "subarray exceeds array support" else
  { support = sup; indices = Range rng }
;;

let indices sup ia =
  let len = Uarray.length sup in
  if Uarray.exists (fun i -> i > len) ia then
    sa_invalid_arg "indices" "subarray exceeds array support" else
  { support = sup; indices = Indices ia; }
;;

let of_t { support = sup; indices = indices; } = (sup, indices);;
